#! /bin/bash
MYNAME=$(basename $0)
MYDIR=$(dirname $0)
MYDIR=$(readlink -m "$MYDIR")
VERSION="0.0.4"
VERBOSE=""
LOGFILE=""
SERVER=""
SSH="ssh -o UserKnownHostsFile=/dev/null -o StrictHostKeyChecking=no -o LogLevel=quiet"

export LC_ALL=C

#
# Prints the software version and exits.
#
function printVersionAndExit()
{
    echo "$MYNAME Version $VERSION on $(hostname)" >&2
    exit 0
}

#
# $*: the error message
#
#
# Prints an error message to the standard error. The text will not mixed up with
# the data that is printed to the standard output.
#
function printError()
{
    local datestring=$(date "+%Y-%m-%d %H:%M:%S")

    echo -e "$MYNAME($$) $*" >&2

    if [ "$LOGFILE" ]; then
        echo -e "$datestring ERROR $MYNAME($$) $*" >>"$LOGFILE"
    fi
}

#
# $*: the message
#
# Prints all the arguments but only if the program is in the verbose mode.
#
function printVerbose()
{
    local datestring=$(date "+%Y-%m-%d %H:%M:%S")

    if [ "$VERBOSE" == "true" ]; then
        echo -e "$MYNAME($$) $*" >&2
    fi

    if [ "$LOGFILE" ]; then
        echo -e "$datestring DEBUG $MYNAME($$) $*" >>"$LOGFILE"
    fi
}

function printHelpAndExit()
{
cat <<EOF
Usage:
  $MYNAME [OPTION]... 

  $MYNAME - Installs the pip scripts on test servers. 

 -h, --help           Print this help and exit.
 -v, --version        Print version information and exit.
 --verbose            Print more messages.
 --log-file=FILE      Store all the messages in the given file too.
 --server=SERVER      Name of the server or servers where we install.
 --blades             Install on blade01 through blade10.

EXAMPLE

 ./$MYNAME --server="core1,host01"

EOF
    exit 0
}

ARGS=$(\
    getopt \
        -o hvs:c:l \
        -l "help,verbose,version,log-file:,server:,blades" \
        -- "$@")

if [ $? -ne 0 ]; then
    exit 6
fi

eval set -- "$ARGS"
while true; do
    case "$1" in
        -h|--help)
            shift
            printHelpAndExit
            ;;

        --verbose)
            shift
            VERBOSE="true"
            VERBOSE_OPTION="--verbose"
            ;;

        -v|--version)
            shift
            VERSION_OPTION="--version"
            ;;

        --log-file)
            shift
            LOGFILE=$(readlink -m "$1")
            shift
            ;;

        --server)
            shift
            SERVER=$(echo "$1" | tr ',' ' ')
            shift
            ;;

        --blades)
            shift
            if [ -f "$HOME/.pip/blades" ]; then
                SERVER=$(cat "$HOME/.pip/blades" | tr ',' ' ')
            else
                SERVER="blade01 blade02 blade03 blade04 blade05 "
                SERVER+="blade06 blade07 blade08 blade09 blade10"
                echo "$SERVER" >"$HOME/.pip/blades"
            fi
            ;;

        --)
            shift
            break
            ;;

        *)
            ;;
    esac
done

if [ -z "$SERVER" ]; then
    printError "Server name is not specified."
    printError "Use the --server command line option to set it."
    exit 6
fi

#
# Going through the servers and installing the pip scripts on all of them.
#
for server in $SERVER; do
    echo "Installing on host '$server'."

    $SSH $server -- mkdir install_sh_tmp
    scp pip-* utilityfunctions.sh "${server}:install_sh_tmp/"
    
    scp cloud_credentials.json "${server}:install_sh_tmp/"
    scp pip-* utilityfunctions.sh "${server}:install_sh_tmp/"

    $SSH $server -- \
        sudo cp -vf install_sh_tmp/pip-* /usr/bin
    
    $SSH $server -- \
        sudo cp -vf install_sh_tmp/utilityfunctions.sh /usr/bin
    
    $SSH $server -- \
        sudo mkdir -p /var/lib/cmon

    $SSH $server -- \
        sudo cp install_sh_tmp/cloud_credentials.json /var/lib/cmon

    $SSH $server -- \
        rm -rvf install_sh_tmp
done
